#!/usr/bin/env python3

from mosq_test_helper import *

port = mosq_test.get_lib_port()


resp_topic = "response/topic"
pub_topic = "request/topic"

rc = 1
keepalive = 60
connect1_packet = mosq_test.gen_connect("request-test", keepalive=keepalive, proto_ver=5)
connect2_packet = mosq_test.gen_connect("response-test", keepalive=keepalive, proto_ver=5)
connack_packet = mosq_test.gen_connack(rc=0, proto_ver=5)

mid = 1
subscribe1_packet = mosq_test.gen_subscribe(mid, resp_topic, 0, proto_ver=5)
subscribe2_packet = mosq_test.gen_subscribe(mid, pub_topic, 0, proto_ver=5)
suback_packet = mosq_test.gen_suback(mid, 0, proto_ver=5)


props = mqtt5_props.gen_string_prop(mqtt5_props.PROP_RESPONSE_TOPIC, resp_topic)
publish1_packet = mosq_test.gen_publish(pub_topic, qos=0, payload="action", proto_ver=5, properties=props)

publish2_packet = mosq_test.gen_publish(resp_topic, qos=0, payload="a response", proto_ver=5)


sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
sock.settimeout(10)
sock.bind(('', port))
sock.listen(5)

env = dict(os.environ)
env['LD_LIBRARY_PATH'] = '../../lib:../../lib/cpp'
try:
    pp = env['PYTHONPATH']
except KeyError:
    pp = ''
env['PYTHONPATH'] = '../../lib/python:'+pp
client1 = mosq_test.start_client(filename="03-request-response-1.log", cmd=["c/03-request-response-1.test"], env=env, port=port)

try:
    (conn1, address) = sock.accept()
    conn1.settimeout(10)

    client2 = mosq_test.start_client(filename="03-request-response-2.log", cmd=["c/03-request-response-2.test"], env=env, port=port)
    (conn2, address) = sock.accept()
    conn2.settimeout(10)

    mosq_test.do_receive_send(conn1, connect1_packet, connack_packet, "connect1")
    mosq_test.do_receive_send(conn2, connect2_packet, connack_packet, "connect2")

    mosq_test.do_receive_send(conn1, subscribe1_packet, suback_packet, "subscribe1")
    mosq_test.do_receive_send(conn2, subscribe2_packet, suback_packet, "subscribe2")

    mosq_test.expect_packet(conn1, "publish1", publish1_packet)
    conn2.send(publish1_packet)

    mosq_test.expect_packet(conn2, "publish2", publish2_packet)
    rc = 0

    conn1.close()
    conn2.close()
except mosq_test.TestError:
    pass
finally:
    client1.terminate()
    client1.wait()
    client2.terminate()
    client2.wait()
    if rc:
        (stdo, stde) = client1.communicate()
        print(stde)
        (stdo, stde) = client2.communicate()
        print(stde)
    sock.close()

exit(rc)



#!/usr/bin/env python3

# Test whether a client subscribed to a topic receives its own message sent to that topic.
# And whether the no-local option is persisted.

from mosq_test_helper import *

def write_config(filename, port):
    with open(filename, 'w') as f:
        f.write("port %d\n" % (port))
        f.write("allow_anonymous true\n")
        f.write("persistence true\n")
        f.write("persistence_file mosquitto-%d.db\n" % (port))

port = mosq_test.get_port()
conf_file = os.path.basename(__file__).replace('.py', '.conf')
write_config(conf_file, port)

rc = 1
keepalive = 60
connect_packet = mosq_test.gen_connect(
    "persistent-subscription-test", keepalive=keepalive, clean_session=False, proto_ver=5, session_expiry=60
)
connack_packet = mosq_test.gen_connack(rc=0, proto_ver=5)
connack_packet2 = mosq_test.gen_connack(rc=0, flags=1, proto_ver=5)  # session present

mid = 1
subscribe1_packet = mosq_test.gen_subscribe(mid, "subpub/nolocal", 5, proto_ver=5)
suback1_packet = mosq_test.gen_suback(mid, 1, proto_ver=5)

mid = 2
subscribe2_packet = mosq_test.gen_subscribe(mid, "subpub/local", 1, proto_ver=5)
suback2_packet = mosq_test.gen_suback(mid, 1, proto_ver=5)

mid = 1
publish1_packet = mosq_test.gen_publish("subpub/nolocal", qos=1, mid=mid, payload="message", proto_ver=5)
puback1_packet = mosq_test.gen_puback(mid, proto_ver=5)

mid = 2
publish2s_packet = mosq_test.gen_publish("subpub/local", qos=1, mid=mid, payload="message", proto_ver=5)
puback2s_packet = mosq_test.gen_puback(mid, proto_ver=5)

mid = 1
publish2a_packet = mosq_test.gen_publish("subpub/local", qos=1, mid=mid, payload="message", proto_ver=5)
puback2a_packet = mosq_test.gen_puback(mid, proto_ver=5)

mid = 2
publish2b_packet = mosq_test.gen_publish("subpub/local", qos=1, mid=mid, payload="message", proto_ver=5)
puback2b_packet = mosq_test.gen_puback(mid, proto_ver=5)

if os.path.exists('mosquitto-%d.db' % (port)):
    os.unlink('mosquitto-%d.db' % (port))

broker = mosq_test.start_broker(filename=os.path.basename(__file__), use_conf=True, port=port)

(stdo1, stde1) = ("", "")
try:
    sock = mosq_test.do_client_connect(connect_packet, connack_packet, timeout=20, port=port)
    mosq_test.do_send_receive(sock, subscribe1_packet, suback1_packet, "suback1")
    mosq_test.do_send_receive(sock, subscribe2_packet, suback2_packet, "suback2")

    mosq_test.do_send_receive(sock, publish1_packet, puback1_packet, "puback1a")
    sock.send(publish2s_packet)
    mosq_test.receive_unordered(sock, puback2s_packet, publish2a_packet, "puback2a/publish2a")

    sock.send(puback2a_packet)

    broker.terminate()
    broker.wait()
    (stdo1, stde1) = broker.communicate()
    sock.close()

    broker = mosq_test.start_broker(filename=os.path.basename(__file__), use_conf=True, port=port)

    sock = mosq_test.do_client_connect(connect_packet, connack_packet2, timeout=20, port=port)

    mosq_test.do_send_receive(sock, publish1_packet, puback1_packet, "puback1b")
    sock.send(publish2s_packet)
    mosq_test.receive_unordered(sock, puback2s_packet, publish2b_packet, "puback2b/publish2b")

    rc = 0

    sock.close()
except mosq_test.TestError:
    pass
finally:
    os.remove(conf_file)
    broker.terminate()
    broker.wait()
    (stdo, stde) = broker.communicate()
    if rc:
        print(stde.decode('utf-8'))
    if os.path.exists('mosquitto-%d.db' % (port)):
        os.unlink('mosquitto-%d.db' % (port))


exit(rc)




#!/usr/bin/env python3

from mosq_test_helper import *


def do_test(proto_ver):
    rc = 1
    mid = 3265
    keepalive = 60
    connect_packet = mosq_test.gen_connect("pub-qos2-disco-test", keepalive=keepalive, clean_session=False, proto_ver=proto_ver, session_expiry=60)
    connack1_packet = mosq_test.gen_connack(flags=0, rc=0, proto_ver=proto_ver)

    if proto_ver == 3:
        connack2_packet = mosq_test.gen_connack(flags=0, rc=0, proto_ver=proto_ver)
    else:
        connack2_packet = mosq_test.gen_connack(flags=1, rc=0, proto_ver=proto_ver)

    helper_connect_packet = mosq_test.gen_connect("pub-qos2-disco-helper", keepalive=keepalive, clean_session=True, proto_ver=proto_ver)
    helper_connack_packet = mosq_test.gen_connack(flags=0, rc=0, proto_ver=proto_ver)
    subscribe_packet = mosq_test.gen_subscribe(mid, "qos2/disconnect/test", 2, proto_ver=proto_ver)
    suback_packet = mosq_test.gen_suback(mid, 2, proto_ver=proto_ver)

    mid = 1
    publish_packet = mosq_test.gen_publish("qos2/disconnect/test", qos=2, mid=mid, payload="disconnect-message", proto_ver=proto_ver)
    publish_dup_packet = mosq_test.gen_publish("qos2/disconnect/test", qos=2, mid=mid, payload="disconnect-message", dup=True, proto_ver=proto_ver)
    pubrec_packet = mosq_test.gen_pubrec(mid, proto_ver=proto_ver)
    pubrel_packet = mosq_test.gen_pubrel(mid, proto_ver=proto_ver)
    if proto_ver == 3:
        pubrel_dup_packet = mosq_test.gen_pubrel(mid, dup=True, proto_ver=proto_ver)
    else:
        pubrel_dup_packet = mosq_test.gen_pubrel(mid, dup=False, proto_ver=proto_ver)
    pubcomp_packet = mosq_test.gen_pubcomp(mid, proto_ver=proto_ver)

    port = mosq_test.get_port()
    broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=port)

    try:
        # Add a subscriber, so we ensure that the QoS 2 flow must be completed
        helper = mosq_test.do_client_connect(helper_connect_packet, helper_connack_packet, port=port)
        mosq_test.do_send_receive(helper, subscribe_packet, suback_packet)

        sock = mosq_test.do_client_connect(connect_packet, connack1_packet, port=port)

        mosq_test.do_send_receive(sock, publish_packet, pubrec_packet, "pubrec")

        # We're now going to disconnect and pretend we didn't receive the pubrec.
        sock.close()

        sock = mosq_test.do_client_connect(connect_packet, connack2_packet, port=port, connack_error="connack 2")
        sock.send(publish_dup_packet)

        mosq_test.expect_packet(sock, "pubrec", pubrec_packet)
        mosq_test.do_send_receive(sock, pubrel_packet, pubcomp_packet, "pubcomp")

        # Again, pretend we didn't receive this pubcomp
        sock.close()

        sock = mosq_test.do_client_connect(connect_packet, connack2_packet, port=port)
        mosq_test.do_send_receive(sock, pubrel_dup_packet, pubcomp_packet, "pubcomp")

        rc = 0

        sock.close()
        helper.close()
    except mosq_test.TestError:
        pass
    finally:
        broker.terminate()
        broker.wait()
        (stdo, stde) = broker.communicate()
        if rc:
            print(stde.decode('utf-8'))
            print("proto_ver=%d" % (proto_ver))
            exit(rc)


do_test(proto_ver=3)
do_test(proto_ver=4)
do_test(proto_ver=5)
exit(0)




#!/usr/bin/env python3

# This tests the default ACL type access behaviour for when no ACL matches.

from mosq_test_helper import *
import json
import shutil

def write_config(filename, port):
    with open(filename, 'w') as f:
        f.write("listener %d\n" % (port))
        f.write("allow_anonymous false\n")
        f.write("plugin ../../plugins/dynamic-security/mosquitto_dynamic_security.so\n")
        f.write("plugin_opt_config_file %d/dynamic-security.json\n" % (port))

def command_check(sock, command_payload, expected_response):
    command_packet = mosq_test.gen_publish(topic="$CONTROL/dynamic-security/v1", qos=0, payload=json.dumps(command_payload))
    sock.send(command_packet)
    response = json.loads(mosq_test.read_publish(sock))
    if response != expected_response:
        print("Expected: %s" % (expected_response))
        print("Received: %s" % (response))
        raise ValueError(response)



port = mosq_test.get_port()
conf_file = os.path.basename(__file__).replace('.py', '.conf')
write_config(conf_file, port)

add_client_command = { "commands": [{
    "command": "createClient", "username": "user_one",
    "password": "password", "clientid": "cid",
    "correlationData": "2" }]
}
add_client_response = {'responses': [{'command': 'createClient', 'correlationData': '2'}]}

get_access_command = { "commands": [{"command": "getDefaultACLAccess", "correlationData": "3" }]}
get_access_response = {'responses': [
    {
        "command": "getDefaultACLAccess",
        'data': {'acls': [
            {'acltype': 'publishClientSend', 'allow': False},
            {'acltype': 'publishClientReceive', 'allow': True},
            {'acltype': 'subscribe', 'allow': False},
            {'acltype': 'unsubscribe', 'allow': True}
        ]},
        "correlationData": "3"
    }]
}

allow_subscribe_command = { "commands": [
    {
        "command": "setDefaultACLAccess",
        "acls":[
            { "acltype": "subscribe", "allow": True }
		],
        "correlationData": "4" }
    ]
}
allow_subscribe_response = {'responses': [{'command': 'setDefaultACLAccess', 'correlationData': '4'}]}

allow_publish_send_command = { "commands": [
    {
        "command": "setDefaultACLAccess",
        "acls":[
            { "acltype": "publishClientSend", "allow": True }
		],
        "correlationData": "5" }
    ]
}
allow_publish_send_response = {'responses': [{'command': 'setDefaultACLAccess', 'correlationData': '5'}]}

allow_publish_recv_command = { "commands": [
    {
        "command": "setDefaultACLAccess",
        "acls":[
            { "acltype": "publishClientReceive", "allow": False }
		],
        "correlationData": "6" }
    ]
}
allow_publish_recv_response = {'responses': [{'command': 'setDefaultACLAccess', 'correlationData': '6'}]}

allow_unsubscribe_command = { "commands": [
    {
        "command": "setDefaultACLAccess",
        "acls":[
            { "acltype": "unsubscribe", "allow": False }
		],
        "correlationData": "7" }
    ]
}
allow_unsubscribe_response = {'responses': [{'command': 'setDefaultACLAccess', 'correlationData': '7'}]}

rc = 1
keepalive = 10
connect_packet_admin = mosq_test.gen_connect("ctrl-test", keepalive=keepalive, username="admin", password="admin")
connack_packet_admin = mosq_test.gen_connack(rc=0)

mid = 2
subscribe_packet_admin = mosq_test.gen_subscribe(mid, "$CONTROL/dynamic-security/#", 1)
suback_packet_admin = mosq_test.gen_suback(mid, 1)

connect_packet = mosq_test.gen_connect("cid", keepalive=keepalive, username="user_one", password="password", proto_ver=5)
connack_packet = mosq_test.gen_connack(rc=0, proto_ver=5)

mid = 3
subscribe_packet = mosq_test.gen_subscribe(mid, "topic", 0, proto_ver=5)
suback_packet_fail = mosq_test.gen_suback(mid, mqtt5_rc.MQTT_RC_NOT_AUTHORIZED, proto_ver=5)
suback_packet_success = mosq_test.gen_suback(mid, 0, proto_ver=5)

mid = 4
unsubscribe_packet = mosq_test.gen_unsubscribe(mid, "topic", proto_ver=5)
unsuback_packet_fail = mosq_test.gen_unsuback(mid, mqtt5_rc.MQTT_RC_NOT_AUTHORIZED, proto_ver=5)
unsuback_packet_success = mosq_test.gen_unsuback(mid, proto_ver=5)

mid = 5
publish_packet = mosq_test.gen_publish(topic="topic", mid=mid, qos=1, payload="message", proto_ver=5)
puback_packet_fail = mosq_test.gen_puback(mid, proto_ver=5, reason_code=mqtt5_rc.MQTT_RC_NOT_AUTHORIZED)
puback_packet_success = mosq_test.gen_puback(mid, proto_ver=5)

publish_packet_recv = mosq_test.gen_publish(topic="topic", qos=0, payload="message", proto_ver=5)

try:
    os.mkdir(str(port))
    shutil.copyfile("dynamic-security-init.json", "%d/dynamic-security.json" % (port))
except FileExistsError:
    pass

broker = mosq_test.start_broker(filename=os.path.basename(__file__), use_conf=True, port=port)

try:
    sock = mosq_test.do_client_connect(connect_packet_admin, connack_packet_admin, timeout=5, port=port)
    mosq_test.do_send_receive(sock, subscribe_packet_admin, suback_packet_admin, "admin suback")

    # Add client
    command_check(sock, add_client_command, add_client_response)
    command_check(sock, get_access_command, get_access_response)

    csock = mosq_test.do_client_connect(connect_packet, connack_packet, timeout=5, port=port)

    # Subscribe should fail because default access is deny
    mosq_test.do_send_receive(csock, subscribe_packet, suback_packet_fail, "suback fail")

    # Set default subscribe access to allow
    command_check(sock, allow_subscribe_command, allow_subscribe_response)

    # Subscribe should succeed because default access is now allowed
    mosq_test.do_send_receive(csock, subscribe_packet, suback_packet_success, "suback success")

    # Publish should fail because publishClientSend default is denied
    mosq_test.do_send_receive(csock, publish_packet, puback_packet_fail, "puback fail")

    # Set default publish send access to allow
    command_check(sock, allow_publish_send_command, allow_publish_send_response)

    # Publish should now succeed because publishClientSend default is allow
    # We also receive the message because publishClientReceive default is allow.
    csock.send(publish_packet)
    mosq_test.receive_unordered(csock, puback_packet_success, publish_packet_recv, "puback success / publish recv")

    # Set default publish receive access to deny
    command_check(sock, allow_publish_recv_command, allow_publish_recv_response)

    # Publish should succeed because publishClientSend default is allow
    # We should *not* receive the publish because it has been disabled.
    mosq_test.do_send_receive(csock, publish_packet, puback_packet_success, "puback success")
    mosq_test.do_ping(csock)

    # Unsubscribe should succeed because default access is allowed
    mosq_test.do_send_receive(csock, unsubscribe_packet, unsuback_packet_success, "unsuback success")

    # Set default unsubscribe access to allow
    command_check(sock, allow_unsubscribe_command, allow_unsubscribe_response)

    # Subscribe should succeed because default access is allowed
    mosq_test.do_send_receive(csock, subscribe_packet, suback_packet_success, "suback success 2")

    # Unsubscribe should fail because default access is no longer allowed
    mosq_test.do_send_receive(csock, unsubscribe_packet, unsuback_packet_fail, "unsuback fail")

    csock.close()

    rc = 0

    sock.close()
except mosq_test.TestError:
    pass
finally:
    os.remove(conf_file)
    try:
        os.remove(f"{port}/dynamic-security.json")
    except FileNotFoundError:
        pass
    os.rmdir(f"{port}")
    broker.terminate()
    broker.wait()
    (stdo, stde) = broker.communicate()
    if rc:
        print(stde.decode('utf-8'))


exit(rc)



#!/usr/bin/env python

import gnomeapplet
import gtk
import mosquitto
import sys

class CurrentCostMQTT(gnomeapplet.Applet):
    def on_message(self, mosq, obj, msg):
        # Message format is "power"
        self.label.set_text(msg.payload+"W")

    def set_label(self, val):
        self.label.set_text(val)

    def on_change_background(self, applet, type, color, pixmap):
        applet.set_style(None)
        applet.modify_style(gtk.RcStyle())

        if type == gnomeapplet.COLOR_BACKGROUND:
            applet.modify_bg(gtk.STATE_NORMAL, color)
        elif type == gnomeapplet.PIXMAP_BACKGROUND:
            style = applet.get_style().copy()
            style.bg_pixmap[gtk.STATE_NORMAL] = pixmap
            applet.set_style(style)

    def show_menu(self, widget, event):
        print "menu"

    def __init__(self, applet, iid):
        self.applet = applet
        self.label = gtk.Label("0W")
        self.event_box = gtk.EventBox()
        self.event_box.add(self.label)
        self.event_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.event_box.connect("button_press_event", self.show_menu)
        self.applet.add(self.event_box)
        self.applet.set_background_widget(applet)
        self.applet.show_all()
        self.mosq = mosquitto.Mosquitto()
        self.mosq.on_message = self.on_message
        self.mosq.connect("localhost")
        self.mosq.loop_start()
        self.mosq.subscribe("sensors/cc128/ch1", 0)
        self.applet.connect('change-background', self.on_change_background)

def CurrentCostMQTT_factory(applet, iid):
    CurrentCostMQTT(applet, iid)
    return gtk.TRUE

if len(sys.argv) == 2:
    if sys.argv[1] == "-d": #Debug mode
        main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        main_window.set_title("Python Applet")
        main_window.connect("destroy", gtk.main_quit)
        app = gnomeapplet.Applet()
        CurrentCostMQTT_factory(app,None)
        app.reparent(main_window)
        main_window.show_all()
        gtk.main()
        sys.exit()

if __name__ == '__main__':
    gnomeapplet.bonobo_factory("OAFIID:CurrentCostMQTT_Factory", gnomeapplet.Applet.__gtype__, "MQTT", "0", CurrentCostMQTT_factory)




#!/usr/bin/env python3

# Test whether a client subscribed to a topic receives its own message sent to that topic, for long topics.

from mosq_test_helper import *

def do_test(topic, succeeds):
    rc = 1
    mid = 53
    keepalive = 60
    connect_packet = mosq_test.gen_connect("subpub-qos0-test", keepalive=keepalive)
    connack_packet = mosq_test.gen_connack(rc=0)

    subscribe_packet = mosq_test.gen_subscribe(mid, topic, 0)
    suback_packet = mosq_test.gen_suback(mid, 0)

    publish_packet = mosq_test.gen_publish(topic, qos=0, payload="message")

    port = mosq_test.get_port()
    broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=port)

    try:
        sock = mosq_test.do_client_connect(connect_packet, connack_packet, timeout=20, port=port)

        if succeeds == True:
            mosq_test.do_send_receive(sock, subscribe_packet, suback_packet, "suback")
            mosq_test.do_send_receive(sock, publish_packet, publish_packet, "publish")
        else:
            mosq_test.do_send_receive(sock, subscribe_packet, b"", "suback")

        rc = 0

        sock.close()
    except mosq_test.TestError:
        pass
    finally:
        broker.terminate()
        broker.wait()
        (stdo, stde) = broker.communicate()
        if rc:
            print(stde.decode('utf-8'))
            exit(rc)

do_test("/"*200, True) # 200 max hierarchy limit
do_test("abc/"*199+"d", True) # 200 max hierarchy limit, longer overall string than 200

do_test("/"*201, False) # Exceeds 200 max hierarchy limit
do_test("abc/"*201+"d", False) # Exceeds 200 max hierarchy limit, longer overall string than 200


exit(0)




#!/usr/bin/env python3

import paho.mqtt.client as paho
import random
import sys
import time

# This is a client that carries out randomised behaviour. It is intended for
# use with the local config file. This file has multiple listeners configured:
#   * 1883 - unencrypted MQTT over TCP with no authentication
#   * 1884 - unencrypted MQTT over TCP with password authentication
#   * 1885 - unencrypted MQTT over TCP with plugin authentication
#   * 1886 - unencrypted MQTT over TCP with password and plugin authentication
#
#   * 8883 - encrypted MQTT over TCP with no authentication
#   * 8884 - encrypted MQTT over TCP with password authentication
#   * 8885 - encrypted MQTT over TCP with plugin authentication
#   * 8886 - encrypted MQTT over TCP with password and plugin authentication
#
#   * 8000 - unencrypted MQTT over WebSockets with no authentication
#   * 8001 - unencrypted MQTT over WebSockets with password authentication
#   * 8002 - unencrypted MQTT over WebSockets with plugin authentication
#   * 8003 - unencrypted MQTT over WebSockets with password and plugin authentication
#
#   * 4430 - encrypted MQTT over WebSockets with no authentication
#   * 4431 - encrypted MQTT over WebSockets with password authentication
#   * 4432 - encrypted MQTT over WebSockets with plugin authentication
#   * 4433 - encrypted MQTT over WebSockets with password and plugin authentication
#
# The client randomly picks:
#   * A port out of the list
#   * Whether to use encryption
#   * Whether to use WebSockets
#   * Clean start or not
#   * Session expiry interval or not
#   * QoS to use when subscribing - topics "outgoing/[client id]/message" and "response/#"
#   * Lifetime of connection
# On a per publish message basis it chooses:
#   * QoS of message
#   * Topic of message "outgoing/[0-max client]/message"
#   * Retain
#   * Interval until next outgoing message

ports = [
        {"port":1883, "tls":False, "transport":"tcp", "auth":False},
        {"port":1884, "tls":False, "transport":"tcp", "auth":True},
        {"port":1885, "tls":False, "transport":"tcp", "auth":True},
        {"port":1886, "tls":False, "transport":"tcp", "auth":True},

        {"port":8883, "tls":True, "transport":"tcp", "auth":False},
        {"port":8884, "tls":True, "transport":"tcp", "auth":True},
        {"port":8885, "tls":True, "transport":"tcp", "auth":True},
        {"port":8886, "tls":True, "transport":"tcp", "auth":True},

        {"port":8000, "tls":False, "transport":"websockets", "auth":False},
        {"port":8001, "tls":False, "transport":"websockets", "auth":True},
        {"port":8002, "tls":False, "transport":"websockets", "auth":True},
        {"port":8003, "tls":False, "transport":"websockets", "auth":True},

        {"port":4430, "tls":True, "transport":"websockets", "auth":False},
        {"port":4431, "tls":True, "transport":"websockets", "auth":True},
        {"port":4432, "tls":True, "transport":"websockets", "auth":True},
        {"port":4433, "tls":True, "transport":"websockets", "auth":True},
        ]

booleans = [True, False]
qos_values = [0, 1, 2]


def on_connect(client, userdata, flags, rc):
    global running
    if rc == 0:
        client.subscribe("response/#", subscribe_qos)
        client.subscribe("outgoing/%s/message" % (client_id), subscribe_qos)
    else:
        running = False


def on_message(client, userdata, msg):
    pass


def on_publish(client, userdata, mid):
    pass


def on_disconnect(client, userdata, rc):
    running = False


def do_publish(client):
    retain = random.choice(booleans)
    qos = random.choice(qos_values)
    topic = "outgoing/%d/message" % (random.uniform(1, 1000))
    payload = "message"

    client.publish(topic, payload, qos, retain)

    next_publish_time = time.time() + random.uniform(0.1, 2.0)


def main():
    global running
    global lifetime

    mqttc = paho.Client(client_id, clean_session=clean_start, protocol=protocol, transport=transport)
    mqttc.on_message = on_message
    mqttc.on_publish = on_publish
    mqttc.on_connect = on_connect
    mqttc.on_disconnect = on_disconnect
    if auth and random.choice(booleans):
        if random.choice(booleans):
            mqttc.username_pw_set("test", "password")
        else:
            mqttc.username_pw_set("bad", "bad")

    if use_tls:
        mqttc.tls_set(ca_certs="../ssl/all-ca.crt")

    mqttc.connect("localhost", port)
    mqttc.loop_start()

    while running:
        time.sleep(0.1)
        now = time.time()
        if now > next_publish_time:
            do_publish(mqttc)
        if now > lifetime:
            if random.choice(booleans):
                mqttc.disconnect()
                lifetime += 5.0
            else:
                running = False


p = random.choice(ports)
port = p["port"]
use_tls = p["tls"]
transport = p["transport"]
auth = p["auth"]

client_id = "cid"+sys.argv[1]
clean_start = random.choice(booleans)
subscribe_qos = random.choice(qos_values)
protocol = paho.MQTTv311
next_publish_time = time.time() + random.uniform(0.1, 2.0)
lifetime = time.time() + random.uniform(5.0, 10.0)
running = True

main()



#!/usr/bin/env python3

# Test whether a clean session client has a QoS 1 message queued for it.

from mosq_test_helper import *

def helper(port):
    connect_packet = mosq_test.gen_connect("test-helper", keepalive=60)
    connack_packet = mosq_test.gen_connack(rc=0)

    mid = 128
    publish_packet = mosq_test.gen_publish("qos1/clean_session/test", qos=1, mid=mid, payload="clean-session-message")
    puback_packet = mosq_test.gen_puback(mid)

    sock = mosq_test.do_client_connect(connect_packet, connack_packet, port=port)
    mosq_test.do_send_receive(sock, publish_packet, puback_packet, "puback")

    sock.close()


def do_test(proto_ver):
    rc = 1
    mid = 109
    keepalive = 60
    connect_packet = mosq_test.gen_connect("clean-qos2-test", keepalive=keepalive, clean_session=False, proto_ver=proto_ver, session_expiry=60)
    connack1_packet = mosq_test.gen_connack(flags=0, rc=0, proto_ver=proto_ver)
    connack2_packet = mosq_test.gen_connack(flags=1, rc=0, proto_ver=proto_ver)

    disconnect_packet = mosq_test.gen_disconnect(proto_ver=proto_ver)

    subscribe_packet = mosq_test.gen_subscribe(mid, "qos1/clean_session/test", 1, proto_ver=proto_ver)
    suback_packet = mosq_test.gen_suback(mid, 1, proto_ver=proto_ver)

    mid = 1
    publish_packet = mosq_test.gen_publish("qos1/clean_session/test", qos=1, mid=mid, payload="clean-session-message", proto_ver=proto_ver)
    puback_packet = mosq_test.gen_puback(mid, proto_ver=proto_ver)

    port = mosq_test.get_port()
    broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=port)

    try:
        sock = mosq_test.do_client_connect(connect_packet, connack1_packet, port=port)
        mosq_test.do_send_receive(sock, subscribe_packet, suback_packet, "suback")

        sock.send(disconnect_packet)
        sock.close()

        helper(port)

        # Now reconnect and expect a publish message.
        sock = mosq_test.do_client_connect(connect_packet, connack2_packet, timeout=30, port=port)
        mosq_test.expect_packet(sock, "publish", publish_packet)
        sock.send(puback_packet)
        rc = 0

        sock.close()
    except mosq_test.TestError:
        pass
    finally:
        broker.terminate()
        broker.wait()
        (stdo, stde) = broker.communicate()
        if rc:
            print(stde.decode('utf-8'))
            print("proto_ver=%d" % (proto_ver))
            exit(rc)


do_test(proto_ver=4)
do_test(proto_ver=5)
exit(0)




#!/usr/bin/env python3

# Test whether a client responds correctly to a PUBLISH with QoS 1.

# The client should connect to port 1888 with keepalive=60, clean session set,
# and client id publish-qos2-test
# The test will send a CONNACK message to the client with rc=0. Upon receiving
# the CONNACK the client should verify that rc==0.
# The test will send the client a PUBLISH message with topic
# "pub/qos2/receive", payload of "message", QoS=2 and mid=13423. The client
# should handle this as per the spec by sending a PUBREC message.
# The test will not respond to the first PUBREC message, so the client must
# resend the PUBREC message with dup=1. Note that to keep test durations low, a
# message retry timeout of less than 10 seconds is required for this test.
# On receiving the second PUBREC with dup==1, the test will send the correct
# PUBREL message. The client should respond to this with the correct PUBCOMP
# message and then exit with return code=0.

from mosq_test_helper import *

port = mosq_test.get_lib_port()

rc = 1
keepalive = 60
connect_packet = mosq_test.gen_connect("publish-qos2-test", keepalive=keepalive)
connack_packet = mosq_test.gen_connack(rc=0)

disconnect_packet = mosq_test.gen_disconnect()

mid = 13423
publish_packet = mosq_test.gen_publish("pub/qos2/receive", qos=2, mid=mid, payload="message")
pubrec_packet = mosq_test.gen_pubrec(mid)
pubrel_packet = mosq_test.gen_pubrel(mid)
pubcomp_packet = mosq_test.gen_pubcomp(mid)

sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
sock.settimeout(10)
sock.bind(('', port))
sock.listen(5)

client_args = sys.argv[1:]
env = dict(os.environ)
env['LD_LIBRARY_PATH'] = '../../lib:../../lib/cpp'
try:
    pp = env['PYTHONPATH']
except KeyError:
    pp = ''
env['PYTHONPATH'] = '../../lib/python:'+pp
client = mosq_test.start_client(filename=sys.argv[1].replace('/', '-'), cmd=client_args, env=env, port=port)

try:
    (conn, address) = sock.accept()
    conn.settimeout(10)

    mosq_test.do_receive_send(conn, connect_packet, connack_packet, "connect")
    mosq_test.do_send_receive(conn, publish_packet, pubrec_packet, "pubrec")
    mosq_test.do_send_receive(conn, pubrel_packet, pubcomp_packet, "pubcomp")
    rc = 0

    conn.close()
except mosq_test.TestError:
    pass
finally:
    for i in range(0, 5):
        if client.returncode != None:
            break
        time.sleep(0.1)

    try:
        client.terminate()
    except OSError:
        pass

    client.wait()
    sock.close()
    if client.returncode != 0:
        exit(1)

exit(rc)



#!/usr/bin/env python3

# Test whether a client subscribed to a topic receives its own message sent to that topic.
# Does a given property get sent through?
# MQTT v5

from mosq_test_helper import *

def prop_subpub_helper(props_out, props_in, expect_proto_error=False):
    rc = 1
    mid = 53
    keepalive = 60
    connect_packet = mosq_test.gen_connect("subpub-qos0-test", keepalive=keepalive, proto_ver=5)
    connack_packet = mosq_test.gen_connack(rc=0, proto_ver=5)

    subscribe_packet = mosq_test.gen_subscribe(mid, "subpub/qos0", 0, proto_ver=5)
    suback_packet = mosq_test.gen_suback(mid, 0, proto_ver=5)

    publish_packet_out = mosq_test.gen_publish("subpub/qos0", qos=0, payload="message", proto_ver=5, properties=props_out)

    publish_packet_expected = mosq_test.gen_publish("subpub/qos0", qos=0, payload="message", proto_ver=5, properties=props_in)

    disconnect_packet = mosq_test.gen_disconnect(reason_code=mqtt5_rc.MQTT_RC_PROTOCOL_ERROR, proto_ver=5)

    port = mosq_test.get_port()
    broker = mosq_test.start_broker(filename=os.path.basename(__file__), port=port)

    try:
        sock = mosq_test.do_client_connect(connect_packet, connack_packet, timeout=20, port=port)

        mosq_test.do_send_receive(sock, subscribe_packet, suback_packet, "suback")
        if expect_proto_error:
            mosq_test.do_send_receive(sock, publish_packet_out, disconnect_packet, "publish")
        else:
            mosq_test.do_send_receive(sock, publish_packet_out, publish_packet_expected, "publish")

        rc = 0

        sock.close()
    except mosq_test.TestError:
        pass
    finally:
        broker.terminate()
        broker.wait()
        (stdo, stde) = broker.communicate()
        if rc:
            print(stde.decode('utf-8'))

    exit(rc)

